@@ -1,23 +1,33 @@
 #!/usr/bin/env python2.4
 from basetest import BaseTest
-import sys, tempfile, os, shutil
+import sys, tempfile, os, shutil, StringIO
 import unittest
 
 sys.path.insert(0, '..')
-from zeroinstall.injector import writer, model, basedir, reader
+from zeroinstall.injector import writer, model, reader, qdom
+from zeroinstall.support import basedir
+
+test_feed = qdom.parse(StringIO.StringIO("""<interface xmlns='http://zero-install.sourceforge.net/2004/injector/interface'>
+<name>Test</name>
+<summary>for testing</summary>
+<implementation id='sha1=3ce644dc725f1d21cfcf02562c76f375944b266a' version='1'/>
+</interface>
+"""))
 
 class TestWriter(BaseTest):
 	def testFeeds(self):
 		iface = model.Interface('http://test/test')
+		iface._main_feed = model.ZeroInstallFeed(test_feed, local_path = '/Hello')
 		iface.stability_policy = model.developer
-		iface.last_checked = 100
-		iface.feeds.append(model.Feed('http://sys-feed', None, False))
-		iface.feeds.append(model.Feed('http://user-feed', 'Linux-*', True))
+		iface._main_feed.last_checked = 100
+		iface.extra_feeds.append(model.Feed('http://sys-feed', None, False))
+		iface.extra_feeds.append(model.Feed('http://user-feed', 'Linux-*', True))
 		writer.save_interface(iface)
 
 		iface = model.Interface('http://test/test')
 		self.assertEquals(None, iface.stability_policy)
-		reader.update_user_overrides(iface)
+		iface._main_feed = model.ZeroInstallFeed(test_feed, local_path = '/Hello')
+		reader.update_user_overrides(iface, iface._main_feed)
 		self.assertEquals(model.developer, iface.stability_policy)
 		self.assertEquals(100, iface.last_checked)
 		self.assertEquals(None, iface.get_feed('http://sys-feed'))
@@ -28,7 +38,6 @@ class TestWriter(BaseTest):
 
 	def testStoreNothing(self):
 		iface = model.Interface('http://test/test')
-		impl = iface.get_impl('/some/path')
 		writer.save_interface(iface)
 
 		iface = model.Interface('http://test/test')
@@ -37,12 +46,13 @@ class TestWriter(BaseTest):
 		self.assertEquals({}, iface.implementations)
 
 	def testStoreStability(self):
-		iface = model.Interface('http://localhost:8000/Hello')
-		impl = iface.get_impl('sha1=3ce644dc725f1d21cfcf02562c76f375944b266a')
+		iface = model.Interface('http://localhost:8000/Hello.xml')
+		iface._main_feed = model.ZeroInstallFeed(test_feed, local_path = '/Hello.xml')
+		impl = iface.implementations['sha1=3ce644dc725f1d21cfcf02562c76f375944b266a']
 		impl.user_stability = model.developer
 		writer.save_interface(iface)
 
-		iface = model.Interface('http://localhost:8000/Hello')
+		iface = model.Interface('http://localhost:8000/Hello.xml')
 		self.assertEquals(None, iface.stability_policy)
 		reader.update_user_overrides(iface)
 
