@@ -1,9 +1,11 @@
+# Copyright (C) 2008, Thomas Leonard
+# See the README file for details, or visit http://0install.net.
+
 import gtk
 from logging import warn
 import os, sys
 import help_box
-from gui import policy, Template
-from dialog import Dialog, MixedButton, frame
+from dialog import Dialog, MixedButton, frame, Template
 from zeroinstall.injector.model import network_levels
 from zeroinstall.injector import trust, gpg
 from freshness import freshness_levels, Freshness
@@ -14,7 +16,7 @@ tips = gtk.Tooltips()
 SHOW_CACHE = 0
 
 class Preferences:
-	def __init__(self):
+	def __init__(self, policy):
 		widgets = Template('preferences_box')
 
 		self.window = widgets.get_widget('preferences_box')
@@ -65,7 +67,6 @@ class Preferences:
 		self.window.default_widget.grab_focus()
 
 		def response(dialog, resp):
-			import download_box
 			if resp in (gtk.RESPONSE_CLOSE, gtk.RESPONSE_DELETE_EVENT):
 				self.window.destroy()
 			elif resp == gtk.RESPONSE_HELP:
@@ -151,13 +152,12 @@ class KeyList:
 		tv.connect('button-press-event', trusted_keys_button_press)
 
 preferences_box = None
-def show_preferences():
+def show_preferences(policy):
 	global preferences_box
-	if preferences_box is not None:
-		preferences_box.window.present()
-	else:
-		preferences_box = Preferences()
-		preferences_box.window.show()
+	if preferences_box:
+		preferences_box.destroy()
+	preferences_box = Preferences(policy)
+	preferences_box.window.show()
 		
 gui_help = help_box.HelpBox("Zero Install Preferences Help",
 ('Overview', """
