@@ -10,6 +10,7 @@ from optparse import OptionParser
 import logging
 
 from zeroinstall.injector import model, download, autopolicy, namespaces
+from zeroinstall.injector.iface_cache import iface_cache
 
 #def program_log(msg): os.access('MARK: 0launch: ' + msg, os.F_OK)
 #import __main__
@@ -17,7 +18,6 @@ from zeroinstall.injector import model, download, autopolicy, namespaces
 #program_log('0launch ' + ' '.join((sys.argv[1:])))
 
 def _list_interfaces(args):
-	from zeroinstall.injector.iface_cache import iface_cache
 	if len(args) == 0:
 		matches = iface_cache.list_all_interfaces()
 	elif len(args) == 1:
@@ -30,9 +30,10 @@ def _list_interfaces(args):
 	for i in matches:
 		print i
 
-def _import_interface(args):
+def _import_feed(args):
+	from zeroinstall.support import tasks
 	from zeroinstall.injector import gpg, handler, trust
-	from zeroinstall.injector.iface_cache import iface_cache, PendingFeed
+	from zeroinstall.injector.iface_cache import PendingFeed
 	from xml.dom import minidom
 	for x in args:
 		if not os.path.isfile(x):
@@ -48,30 +49,49 @@ def _import_interface(args):
 		logging.info("Importing information about interface %s", iface)
 		signed_data.seek(0)
 
-		def keys_ready():
-			if not iface_cache.update_interface_if_trusted(iface, pending.sigs, pending.new_xml):
-				handler.confirm_trust_keys(iface, pending.sigs, pending.new_xml)
-		trust.trust_db.watchers.append(lambda: keys_ready())
-
 		pending = PendingFeed(uri, signed_data)
-		iface_cache.add_pending(pending)
 
 		handler = handler.Handler()
-		pending.begin_key_downloads(handler, keys_ready)
-		handler.wait_for_downloads()
+
+		def run():
+			keys_downloaded = tasks.Task(pending.download_keys(handler), "download keys")
+			yield keys_downloaded.finished
+			tasks.check(keys_downloaded.finished)
+			if not iface_cache.update_interface_if_trusted(iface, pending.sigs, pending.new_xml):
+				blocker = handler.confirm_trust_keys(iface, pending.sigs, pending.new_xml)
+				if blocker:
+					yield blocker
+					tasks.check(blocker)
+				if not iface_cache.update_interface_if_trusted(iface, pending.sigs, pending.new_xml):
+					raise SafeException("No signing keys trusted; not importing")
+
+		task = tasks.Task(run(), "import feed")
+
+		errors = handler.wait_for_blocker(task.finished)
+		if errors:
+			raise model.SafeException("Errors during download: " + '\n'.join(errors))
 
 def _manage_feeds(options, args):
-	from zeroinstall.injector import iface_cache, writer
-	from xml.dom import minidom
+	from zeroinstall.injector import writer
+	from zeroinstall.injector.handler import Handler
+	from zeroinstall.injector.policy import Policy
+	handler = Handler(dry_run = options.dry_run)
 	if not args: raise UsageError()
 	for x in args:
 		print "Feed '%s':\n" % x
 		x = model.canonical_iface_uri(x)
-		policy = autopolicy.AutoPolicy(x, download_only = True, dry_run = options.dry_run)
+		policy = Policy(x, handler)
 		if options.offline:
 			policy.network_use = model.network_offline
-		policy.recalculate_with_dl()
-		interfaces = policy.get_feed_targets(policy.root)
+
+		feed = iface_cache.get_feed(x)
+		if policy.network_use != model.network_offline and policy.is_stale(feed):
+			blocker = policy.fetcher.download_and_import_feed(x, iface_cache.iface_cache)
+			print "Downloading feed; please wait..."
+			handler.wait_for_blocker(blocker)
+			print "Done"
+
+		interfaces = policy.get_feed_targets(x)
 		for i in range(len(interfaces)):
 			feed = interfaces[i].get_feed(x)
 			if feed:
@@ -98,9 +118,9 @@ def _manage_feeds(options, args):
 		iface = interfaces[i - 1]
 		feed = iface.get_feed(x)
 		if feed:
-			iface.feeds.remove(feed)
+			iface.extra_feeds.remove(feed)
 		else:
-			iface.feeds.append(model.Feed(x, arch = None, user_override = True))
+			iface.extra_feeds.append(model.Feed(x, arch = None, user_override = True))
 		writer.save_interface(iface)
 		print "\nFeed list for interface '%s' is now:" % iface.get_name()
 		if iface.feeds:
@@ -120,6 +140,7 @@ def _normal_mode(options, args):
 			raise UsageError()
 
 	iface_uri = model.canonical_iface_uri(args[0])
+	root_iface = iface_cache.get_interface(iface_uri)
 
 	policy = autopolicy.AutoPolicy(iface_uri,
 				download_only = bool(options.download_only),
@@ -127,8 +148,8 @@ def _normal_mode(options, args):
 				src = options.source)
 
 	if options.before or options.not_before:
-		policy.root_restrictions.append(model.Restriction(model.parse_version(options.before),
-								  model.parse_version(options.not_before)))
+		policy.solver.extra_restrictions[root_iface] = [model.VersionRangeRestriction(model.parse_version(options.before),
+									      		      model.parse_version(options.not_before))]
 
 	if options.offline:
 		policy.network_use = model.network_offline
@@ -139,18 +160,20 @@ def _normal_mode(options, args):
 		if options.main:
 			raise model.SafeException("Can't use --main with --get-selections")
 
-	# Note that need_download() triggers a recalculate()
+	# Note that need_download() triggers a solve
 	if options.refresh or options.gui:
 		# We could run immediately, but the user asked us not to
 		can_run_immediately = False
 	else:
 		can_run_immediately = (not policy.need_download()) and policy.ready
 
-		if options.download_only and policy.stale_feeds:
+		stale_feeds = [feed for feed in policy.solver.feeds_used if policy.is_stale(iface_cache.get_feed(feed))]
+
+		if options.download_only and stale_feeds:
 			can_run_immediately = False
 
 	if can_run_immediately:
-		if policy.stale_feeds:
+		if stale_feeds:
 			if policy.network_use == model.network_offline:
 				logging.debug("No doing background update because we are in off-line mode.")
 			else:
@@ -161,7 +184,11 @@ def _normal_mode(options, args):
 		if options.get_selections:
 			_get_selections(policy)
 		else:
-			policy.execute(args[1:], main = options.main, wrapper = options.wrapper)
+			if not options.download_only:
+				from zeroinstall.injector import run
+				run.execute(policy, args[1:], dry_run = options.dry_run, main = options.main, wrapper = options.wrapper)
+			else:
+				logging.info("Downloads done (download-only mode)")
 			assert options.dry_run or options.download_only
 		return
 
@@ -194,6 +221,10 @@ def _normal_mode(options, args):
 				gui_args.insert(0, '--before')
 			if options.source:
 				gui_args.insert(0, '--source')
+			if options.verbose:
+				gui_args.insert(0, '--verbose')
+				if options.verbose > 1:
+					gui_args.insert(0, '--verbose')
 			sels = _fork_gui(iface_uri, gui_args, prog_args, options)
 			if not sels:
 				sys.exit(1)		# Aborted
@@ -221,7 +252,17 @@ def _fork_gui(iface_uri, gui_args, prog_args, options = None):
 					     bool(options and options.dry_run),
 					     options and options.main)
 	return helpers.get_selections_gui(iface_uri, gui_args, test_callback)
-	
+
+def _download_missing_selections(options, sels):
+	from zeroinstall.injector import fetch
+	from zeroinstall.injector.handler import Handler
+	handler = Handler(dry_run = options.dry_run)
+	fetcher = fetch.Fetcher(handler)
+	blocker = sels.download_missing(iface_cache, fetcher)
+	if blocker:
+		logging.info("Waiting for selected implementations to be downloaded...")
+		handler.wait_for_blocker(blocker)
+
 def _get_selections(policy):
 	import selections
 	doc = selections.Selections(policy).toDOM()
@@ -295,9 +336,10 @@ def main(command_args):
 		elif options.set_selections:
 			from zeroinstall.injector import selections, qdom, run
 			sels = selections.Selections(qdom.parse(file(options.set_selections)))
+			_download_missing_selections(options, sels)
 			run.execute_selections(sels, args, options.dry_run, options.main, options.wrapper)
 		elif getattr(options, 'import'):
-			_import_interface(args)
+			_import_feed(args)
 		elif options.feed:
 			_manage_feeds(options, args)
 		else:
