@@ -13,75 +13,93 @@ To do this, you supply a L{Handler} to the L{policy}.
 import os, sys
 from logging import debug, info, warn
 
+from zeroinstall import NeedDownload
+from zeroinstall.support import tasks
 from zeroinstall.injector import model, download
 from zeroinstall.injector.iface_cache import iface_cache
 
+class NoTrustedKeys(model.SafeException):
+	"""Thrown by L{Handler.confirm_trust_keys} on failure."""
+	pass
+
 class Handler(object):
 	"""
-	This implementation of the handler interface uses the GLib mainloop.
+	This implementation uses the GLib mainloop. Note that QT4 can use the GLib mainloop too.
 
 	@ivar monitored_downloads: dict of downloads in progress
-	@type monitored_downloads: {URL: (error_stream, L{download.Download})}
+	@type monitored_downloads: {URL: L{download.Download}}
+	@ivar n_completed_downloads: number of downloads which have finished for GUIs, etc (can be reset as desired).
+	@type n_completed_downloads: int
+	@ivar total_bytes_downloaded: informational counter for GUIs, etc (can be reset as desired). Updated when download finishes.
+	@type total_bytes_downloaded: int
 	"""
 
-	__slots__ = ['monitored_downloads', '_loop']
+	__slots__ = ['monitored_downloads', '_loop', 'dry_run', 'total_bytes_downloaded', 'n_completed_downloads']
 
-	def __init__(self, mainloop = None):
+	def __init__(self, mainloop = None, dry_run = False):
 		self.monitored_downloads = {}		
 		self._loop = None
+		self.dry_run = dry_run
+		self.n_completed_downloads = 0
+		self.total_bytes_downloaded = 0
 	
 	def monitor_download(self, dl):
 		"""Called when a new L{download} is started.
-		Call L{download.Download.start} to start the download and get the error
-		stream, and then call L{download.Download.error_stream_data} whenever
-		you read any data from it, including nothing (end-of-file), which
-		indicates that the download is finished."""
-		error_stream = dl.start()
-		self.monitored_downloads[dl.url] = (error_stream, dl)
+		This is mainly used by the GUI to display the progress bar."""
+		dl.start()
+		self.monitored_downloads[dl.url] = dl
+		self.downloads_changed()
 
-		import gobject
-		gobject.io_add_watch(error_stream.fileno(),
-				     gobject.IO_IN | gobject.IO_ERR | gobject.IO_HUP,
-				     self._error_stream_ready, dl)
-	
-	def _error_stream_ready(self, fd, cond, dl):
-		debug("Download stream for %s is ready...", dl)
-		errors = os.read(fd, 100)
-		if errors:
-			debug("Read data: %s", errors)
-			dl.error_stream_data(errors)
-			return True
-		else:
-			debug("End-of-stream. Download is finished.")
-			del self.monitored_downloads[dl.url]
+		@tasks.async
+		def download_done_stats():
+			yield dl.downloaded
+			# NB: we don't check for exceptions here; someone else should be doing that
 			try:
-				dl.error_stream_closed()
+				self.n_completed_downloads += 1
+				self.total_bytes_downloaded += dl.get_bytes_downloaded_so_far()
+				del self.monitored_downloads[dl.url]
+				self.downloads_changed()
 			except Exception, ex:
 				self.report_error(ex)
-			if self._loop and not self.monitored_downloads:
-				# Exit from wait_for_downloads if this was the last one
-				debug("Exiting mainloop")
-				self._loop.quit()
-			return False
+		download_done_stats()
+
+	def impl_added_to_store(self, impl):
+		"""Called by the L{fetch.Fetcher} when adding an implementation.
+		The GUI uses this to update its display.
+		@param impl: the implementation which has been added
+		@type impl: L{model.Implementation}
+		"""
+		pass
 	
-	def wait_for_downloads(self):
-		"""Monitor all downloads, waiting until they are complete. This is suitable
-		for use by non-interactive programs."""
+	def downloads_changed(self):
+		"""This is just for the GUI to override to update its display."""
+		pass
+	
+	def wait_for_blocker(self, blocker):
+		"""Run a recursive mainloop until blocker is triggered.
+		@param blocker: event to wait on
+		@type blocker: L{tasks.Blocker}"""
+		if not blocker.happened:
+			import gobject
 
-		import gobject
+			def quitter():
+				yield blocker
+				self._loop.quit()
+			quit = tasks.Task(quitter(), "quitter")
 
-		if self.monitored_downloads:
 			assert self._loop is None	# Avoid recursion
 			self._loop = gobject.MainLoop(gobject.main_context_default())
 			try:
-				debug("Entering mainloop, waiting for %d download(s)", len(self.monitored_downloads))
+				debug("Entering mainloop, waiting for %s", blocker)
 				self._loop.run()
 			finally:
 				self._loop = None
-		else:
-			debug("No downloads in progress, so not waiting")
 
-	def get_download(self, url, force = False):
+			assert blocker.happened, "Someone quit the main loop!"
+
+		tasks.check(blocker)
+	
+	def get_download(self, url, force = False, hint = None):
 		"""Return the Download object currently downloading 'url'.
 		If no download for this URL has been started, start one now (and
 		start monitoring it).
@@ -90,13 +108,16 @@ class Handler(object):
 		a new one.
 		@rtype: L{download.Download}
 		"""
+		if self.dry_run:
+			raise NeedDownload(url)
+
 		try:
-			e, dl = self.monitored_downloads[url]
+			dl = self.monitored_downloads[url]
 			if dl and force:
 				dl.abort()
 				raise KeyError
 		except KeyError:
-			dl = download.Download(url)
+			dl = download.Download(url, hint)
 			self.monitor_download(dl)
 		return dl
 
@@ -106,7 +127,8 @@ class Handler(object):
 		@arg interface: the interface being updated
 		@arg sigs: a list of signatures (from L{gpg.check_stream})
 		@arg iface_xml: the downloaded data (not yet trusted)
-		"""
+		@return: a blocker, if confirmation will happen asynchronously, or None
+		@rtype: L{tasks.Blocker}"""
 		from zeroinstall.injector import trust, gpg
 		assert sigs
 		valid_sigs = [s for s in sigs if isinstance(s, gpg.ValidSig)]
@@ -129,7 +151,7 @@ class Handler(object):
 			i = raw_input("Trust [Y/N] ")
 			if not i: continue
 			if i in 'Nn':
-				raise model.SafeException('Not signed with a trusted key')
+				raise NoTrustedKeys('Not signed with a trusted key')
 			if i in 'Yy':
 				break
 		for key in valid_sigs:
@@ -138,9 +160,12 @@ class Handler(object):
 
 		trust.trust_db.notify()
 	
-	def report_error(self, exception):
+	def report_error(self, exception, tb = None):
 		"""Report an exception to the user.
 		@param exception: the exception to report
 		@type exception: L{SafeException}
+		@param tb: optional traceback
 		@since: 0.25"""
 		warn("%s", exception)
+		#import traceback
+		#traceback.print_exception(exception, None, tb)
