@@ -8,11 +8,11 @@ Save per-interface configuration information.
 import os
 from xml.dom import minidom, XMLNS_NAMESPACE
 
-import basedir
+from zeroinstall.support import basedir
 
-from model import *
+from zeroinstall.injector.model import *
 
-from namespaces import config_site, config_prog, XMLNS_IFACE
+from zeroinstall.injector.namespaces import config_site, config_prog, XMLNS_IFACE
 
 def _add_impl(parent, impl):
 	if impl.user_stability:
@@ -38,15 +38,12 @@ def save_interface(interface):
 	if interface.last_checked:
 		root.setAttribute('last-checked', str(interface.last_checked))
 
-	if interface.last_check_attempt:
-		root.setAttribute('last-check-attempt', str(interface.last_check_attempt))
-
 	impls = interface.implementations.values()
 	impls.sort()
 	for impl in impls:
 		_add_impl(root, impl)
 	
-	for feed in interface.feeds:
+	for feed in interface.extra_feeds:
 		if feed.user_override:
 			elem = doc.createElementNS(XMLNS_IFACE, 'feed')
 			root.appendChild(elem)
