@@ -4,7 +4,7 @@
 # See the README file for details, or visit http://0install.net.
 
 import sys, os
-from zeroinstall.zerostore.manifest import generate_manifest, verify, get_algorithm, copy_tree_with_verify
+from zeroinstall.zerostore.manifest import verify, get_algorithm, copy_tree_with_verify
 from zeroinstall import zerostore, SafeException, support
 
 stores = None
@@ -28,7 +28,7 @@ def do_manifest(args):
 		if '=' in name:
 			alg = get_algorithm(name.split('=', 1)[0])
 		else:
-			alg = get_algorithm('sha1')
+			alg = get_algorithm('sha1new')
 	digest = alg.new_digest()
 	for line in alg.generate_manifest(args[0]):
 		print line
